﻿using System;
using System.Net;
using System.Net.Sockets;

public class DtTestPointDriver
{
    //-------------------------------------------------------------------------
    //  DynamicTracar MaskCode
    //  Please do not change the value.
    //-------------------------------------------------------------------------
    public const int DT_VARIABLE_BIT = 0x02;				/* Memory Dump MASK */
    public const int DT_EVTTRG_BIT = 0x08;				/* EventTrigger MASK */
    //-------------------------------------------------------------------------
    //  value Variables length. never set the value over 256
    //  Please do not change the value.
    //-------------------------------------------------------------------------
    public const int DT_MAX_VAL_SIZE = 256;

    //-------------------------------------------------------------------------
    //  Please set 1 when you add Event Value to Test Point
    //-------------------------------------------------------------------------
    public const int DT_ADD_EVENT_INFO = 0;
    //-------------------------------------------------------------------------
    //  if DT_ADD_EVENT_INFO is 1. Please set 4.
    //-------------------------------------------------------------------------
    public const int DT_EVENT_INFO_SIZE = 0;

    //-------------------------------------------------------------------------
    //  Please set 1 when you add Time Value to Test Point
    //-------------------------------------------------------------------------
    public const int DT_ADD_TIME_INFO = 0;
    //-------------------------------------------------------------------------
    //  if DT_ADD_TIME_INFO is 1. Please set 8.
    //-------------------------------------------------------------------------
    public const int DT_TIME_INFO_SIZE = 0;

    //-------------------------------------------------------------------------
    //  DT_ADD_EVENT_INFO == 1 & DT_ADD_TIME_INFO == 1	-> Normal TP with Event & Time info. Please set 0x15.
    //  DT_ADD_EVENT_INFO == 1 & DT_ADD_TIME_INFO == 0	-> Normal TP with Event info. Please set 0x05.
    //  DT_ADD_EVENT_INFO == 0 & DT_ADD_TIME_INFO == 1	-> Normal TP with Time info. Please set 0x11.
    //  DT_ADD_EVENT_INFO == 0 & DT_ADD_TIME_INFO == 0	-> Normal TP. Please set 0x01.
    //-------------------------------------------------------------------------
    public const int DT_NRMLTP_ID = 0x01;
    //-------------------------------------------------------------------------
    //  DT_ADD_EVENT_INFO == 1 & DT_ADD_TIME_INFO == 1	-> Variable TP with Event & Time info. Please set 0x16.
    //  DT_ADD_EVENT_INFO == 1 & DT_ADD_TIME_INFO == 0	-> Variable TP with Event info. Please set 0x06.
    //  DT_ADD_EVENT_INFO == 0 & DT_ADD_TIME_INFO == 1	-> Variable TP with Time info. Please set 0x12.
    //  DT_ADD_EVENT_INFO == 0 & DT_ADD_TIME_INFO == 0	-> Variable TP. Please set 0x02.
    //-------------------------------------------------------------------------
    public const int DT_VALTP_ID = 0x02;

    //-------------------------------------------------------------------------
    //  DynamicTracar TP size info.
    //  Please do not change the value.
    //-------------------------------------------------------------------------
    public const int DT_HEADER_SIZE = 4;
    public const int DT_NRMLTP_SIZE = DT_HEADER_SIZE + 6 + DT_EVENT_INFO_SIZE + DT_TIME_INFO_SIZE;
    public const int DT_MAX_VALTP_SIZE = DT_NRMLTP_SIZE + 1 + DT_MAX_VAL_SIZE;
    public const int DT_EVENTTP_SIZE = DT_NRMLTP_SIZE;

    //-------------------------------------------------------------------------
    //  DynamicTracar MaskCode
    //-------------------------------------------------------------------------
    static public string DT_IP_ADDRESS = "192.168.10.200";
    static public int DT_PORT = 3000;
    static public Socket socket = null;
    static public IPEndPoint dt_ep;
    static public byte[] buff = new byte[DT_NRMLTP_SIZE];
    static public byte[] val_buff = new byte[DT_MAX_VALTP_SIZE];
    static public object thisLock = new object();

    //-------------------------------------------------------------------------
    //  TestPoint InsertCode Method
    //-------------------------------------------------------------------------
    /* Normal TestPoint Method */
    static public void DtTestPoint(int nBaseAddress, int nStepNumber)
    {
        TpBusOut(nBaseAddress, nStepNumber);
    }
    /* Performance TestPoint Method */
    static public void DtTestPointIdle(int nIdleAddress, int nStepNumber)
    {
        /* Call DynamicTracerDriver */
        TpBusOut(nIdleAddress, nStepNumber);
    }

    /* EventTrigger TestPoint Method */
    static public void DtTestPointEventTrigger(int nBaseAddress, int nStepNumber, int theValue)
    {
        /* Call DynamicTracerDriver */
        TpEventTrigger(nBaseAddress, theValue);
    }

    /* MemoryOutput TestPoint Method */
    static public void DtTestPointValue(int nBaseAddress, int nStepNumber, byte theValue, int nLength)
    {
        int count = 0;
        int packet_size = DT_NRMLTP_SIZE + 1 + 1;

        val_buff[count++] = (byte)0xFF;						/* Header */
        val_buff[count++] = (byte)0xFF;						/* Header */
        val_buff[count++] = (byte)(packet_size);			/* Data Size */
        val_buff[count++] = (byte)((packet_size) >> 8);		/* Data Size */
        val_buff[count++] = (byte)DT_VALTP_ID;				/* Variable TP */
        val_buff[count++] = (byte)(nStepNumber);			/* nStepNumber */
        val_buff[count++] = (byte)(nStepNumber >> 8);		/* nStepNumber */
        val_buff[count++] = (byte)(nBaseAddress | 0x02);	/* nBaseAddress */
        val_buff[count++] = (byte)(nBaseAddress >> 8);		/* nBaseAddress */
        val_buff[count++] = (byte)(nBaseAddress >> 16);		/* nBaseAddress */
        val_buff[count++] = (byte)1;						/* size */
        val_buff[count++] = (byte)theValue;					/* value */
        TP_MemoryOutput(packet_size);
    }
    static public void DtTestPointValue(int nBaseAddress, int nStepNumber, char theValue, int nLength)
    {
        int count = 0;
        int packet_size = DT_NRMLTP_SIZE + 1 + 2;

        val_buff[count++] = (byte)0xFF;						/* Header */
        val_buff[count++] = (byte)0xFF;						/* Header */
        val_buff[count++] = (byte)(packet_size);			/* Data Size */
        val_buff[count++] = (byte)((packet_size) >> 8);		/* Data Size */
        val_buff[count++] = (byte)DT_VALTP_ID;				/* Variable TP */
        val_buff[count++] = (byte)(nStepNumber);			/* nStepNumber */
        val_buff[count++] = (byte)(nStepNumber >> 8);		/* nStepNumber */
        val_buff[count++] = (byte)(nBaseAddress | 0x02);	/* nBaseAddress */
        val_buff[count++] = (byte)(nBaseAddress >> 8);		/* nBaseAddress */
        val_buff[count++] = (byte)(nBaseAddress >> 16);		/* nBaseAddress */
        val_buff[count++] = (byte)2;						/* size */
        val_buff[count++] = (byte)theValue;					/* value */
        val_buff[count++] = (byte)(theValue >> 8);			/* value */
        TP_MemoryOutput(packet_size);
    }
    static public void DtTestPointValue(int nBaseAddress, int nStepNumber, short theValue, int nLength)
    {
        int count = 0;
        int packet_size = DT_NRMLTP_SIZE + 1 + 2;

        val_buff[count++] = (byte)0xFF;						/* Header */
        val_buff[count++] = (byte)0xFF;						/* Header */
        val_buff[count++] = (byte)(packet_size);			/* Data Size */
        val_buff[count++] = (byte)((packet_size) >> 8);		/* Data Size */
        val_buff[count++] = (byte)DT_VALTP_ID;				/* Variable TP */
        val_buff[count++] = (byte)(nStepNumber);			/* nStepNumber */
        val_buff[count++] = (byte)(nStepNumber >> 8);		/* nStepNumber */
        val_buff[count++] = (byte)(nBaseAddress | 0x02);	/* nBaseAddress */
        val_buff[count++] = (byte)(nBaseAddress >> 8);		/* nBaseAddress */
        val_buff[count++] = (byte)(nBaseAddress >> 16);		/* nBaseAddress */
        val_buff[count++] = (byte)2;						/* size */
        val_buff[count++] = (byte)theValue;					/* value */
        val_buff[count++] = (byte)(theValue >> 8);			/* value */
        TP_MemoryOutput(packet_size);
    }
    static public void DtTestPointValue(int nBaseAddress, int nStepNumber, int theValue, int nLength)
    {
        int count = 0;
        int packet_size = DT_NRMLTP_SIZE + 1 + 4;

        val_buff[count++] = (byte)0xFF;						/* Header */
        val_buff[count++] = (byte)0xFF;						/* Header */
        val_buff[count++] = (byte)(packet_size);			/* Data Size */
        val_buff[count++] = (byte)((packet_size) >> 8);		/* Data Size */
        val_buff[count++] = (byte)DT_VALTP_ID;				/* Variable TP */
        val_buff[count++] = (byte)(nStepNumber);			/* nStepNumber */
        val_buff[count++] = (byte)(nStepNumber >> 8);		/* nStepNumber */
        val_buff[count++] = (byte)(nBaseAddress | 0x02);	/* nBaseAddress */
        val_buff[count++] = (byte)(nBaseAddress >> 8);		/* nBaseAddress */
        val_buff[count++] = (byte)(nBaseAddress >> 16);		/* nBaseAddress */
        val_buff[count++] = (byte)4;						/* size */
        val_buff[count++] = (byte)theValue;					/* value */
        val_buff[count++] = (byte)(theValue >> 8);			/* value */
        val_buff[count++] = (byte)(theValue >> 16);			/* value */
        val_buff[count++] = (byte)(theValue >> 24);			/* value */
        TP_MemoryOutput(packet_size);
    }
    static public void DtTestPointValue(int nBaseAddress, int nStepNumber, long theValue, int nLength)
    {
        int count = 0;
        int packet_size = DT_NRMLTP_SIZE + 1 + 8;

        val_buff[count++] = (byte)0xFF;						/* Header */
        val_buff[count++] = (byte)0xFF;						/* Header */
        val_buff[count++] = (byte)(packet_size);			/* Data Size */
        val_buff[count++] = (byte)((packet_size) >> 8);		/* Data Size */
        val_buff[count++] = (byte)DT_VALTP_ID;				/* Variable TP */
        val_buff[count++] = (byte)(nStepNumber);			/* nStepNumber */
        val_buff[count++] = (byte)(nStepNumber >> 8);		/* nStepNumber */
        val_buff[count++] = (byte)(nBaseAddress | 0x02);	/* nBaseAddress */
        val_buff[count++] = (byte)(nBaseAddress >> 8);		/* nBaseAddress */
        val_buff[count++] = (byte)(nBaseAddress >> 16);		/* nBaseAddress */
        val_buff[count++] = (byte)8;						/* size */
        val_buff[count++] = (byte)theValue;					/* value */
        val_buff[count++] = (byte)(theValue >> 8);			/* value */
        val_buff[count++] = (byte)(theValue >> 16);			/* value */
        val_buff[count++] = (byte)(theValue >> 24);			/* value */
        val_buff[count++] = (byte)(theValue >> 32);			/* value */
        val_buff[count++] = (byte)(theValue >> 40);			/* value */
        val_buff[count++] = (byte)(theValue >> 48);			/* value */
        val_buff[count++] = (byte)(theValue >> 56);			/* value */
        TP_MemoryOutput(packet_size);
    }
    static public void DtTestPointValue(int nBaseAddress, int nStepNumber, bool theValue, int nLength)
    {
        int count = 0;
        int packet_size = DT_NRMLTP_SIZE + 1 + 1;

        val_buff[count++] = (byte)0xFF;						/* Header */
        val_buff[count++] = (byte)0xFF;						/* Header */
        val_buff[count++] = (byte)(packet_size);			/* Data Size */
        val_buff[count++] = (byte)((packet_size) >> 8);		/* Data Size */
        val_buff[count++] = (byte)DT_VALTP_ID;				/* Variable TP */
        val_buff[count++] = (byte)(nStepNumber);			/* nStepNumber */
        val_buff[count++] = (byte)(nStepNumber >> 8);		/* nStepNumber */
        val_buff[count++] = (byte)(nBaseAddress | 0x02);	/* nBaseAddress */
        val_buff[count++] = (byte)(nBaseAddress >> 8);		/* nBaseAddress */
        val_buff[count++] = (byte)(nBaseAddress >> 16);		/* nBaseAddress */
        val_buff[count++] = (byte)1;						/* size */
        val_buff[count++] = (byte)(theValue ? 0x01 : 0x00);	/* value */
        TP_MemoryOutput(packet_size);
    }

    //-------------------------------------------------------------------------
    // 
    //-------------------------------------------------------------------------
    /* open file */
    static public void TpBusEthernetInit()
    {
        try
        {
            socket = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);
            IPAddress ip = IPAddress.Parse(DT_IP_ADDRESS);
            dt_ep = new IPEndPoint(ip, DT_PORT);

        }
        catch (Exception)
        {
            Console.WriteLine("Error TpBusOutOpen()");
        }
    }

    /* close file */
    static public void TpBusEthernetFinalize()
    {
        try
        {
            socket.Close();
        }
        catch (Exception)
        {
            Console.WriteLine("Error TpBusOutClose()");
        }
    }

    /* Normal TestPoint Method */
    static public void TpBusOut(int nAddress, int nData)
    {
        if (socket == null)
        {
            TpBusEthernetInit();
        }

        buff[0] = (byte)0xFF;				/* Header */
        buff[1] = (byte)0xFF;				/* Header */
        buff[2] = (byte)DT_NRMLTP_SIZE;		/* Data Size */
        buff[3] = (byte)0x00;				/* Data Size */
        buff[4] = (byte)DT_NRMLTP_ID;		/* Normal TP */
        buff[5] = (byte)(nData);			/* nData */
        buff[6] = (byte)(nData >> 8);		/* nData */
        buff[7] = (byte)(nAddress);			/* nAddress */
        buff[8] = (byte)(nAddress >> 8);	/* nAddress */
        buff[9] = (byte)(nAddress >> 16);	/* nAddress */

        try
        {
            socket.SendTo(buff, 0, 10, SocketFlags.None, dt_ep);
        }
        catch (Exception)
        {
            Console.WriteLine("Error TpBusOut()");
        }
    }

    /* MemoryOutput TestPoint Method (send 'val_buff') */
    static public void TP_MemoryOutput(int packet_size)
    {
        try
        {
            socket.SendTo(val_buff, 0, packet_size, SocketFlags.None, dt_ep);
        }
        catch (Exception)
        {
        }
    }

    /* EventTrigger TestPoint Method */
    static public void TpEventTrigger(int nAddress, int nData)
    {
        /* DynamicTracerDriver */
        TpBusOut(nAddress | DT_EVTTRG_BIT, (int)(nData & 0xfff) << 4);
    }
}